define([
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/keys",
    "dojo/promise/all",
    "dojo/on",
    "dojo/Stateful",
    "dojo/topic",
    "dojo/when",

    // epi
    "epi/dependency",
    "epi/shell/DestroyableByKey",
    "epi-cms/ApplicationSettings",
    "epi-cms/contentediting/command/CreateDraft",
    "epi-cms/core/ContentReference",

    // resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.notifications"
], function (
    // dojo
    declare,
    lang,
    keys,
    all,
    on,
    Stateful,
    topic,
    when,

    // epi
    dependency,
    DestroyableByKey,
    ApplicationSettings,
    CreateDraft,
    ContentReference,

    // resources
    resources
) {
    var projectLinksKey = "projectLinks";

    function createProjectIconEl() {
        var span = document.createElement("span");
        span.classList.add("dijitReset", "dijitInline", "dijitIcon", "epi-objectIcon", "epi-iconProject");
        return span;
    }

    function createMessageContainerEl() {
        var span = document.createElement("span");
        span.classList.add("dijitReset", "dijitInline", "epi-project-notification__content");
        return span;
    }

    return declare([Stateful, DestroyableByKey], {
        // summary:
        //      Show notification if the content is part of a project
        // tags:
        //      internal

        // order: [public] Number
        //      Sort order of notification
        order: 10,

        // _projectService: [private] ProjectService
        //      A service for interacting with projects
        _projectService: null,

        // _createDraftCommand: [private] Command
        //      The create draft command that will be displayed when the current
        //      content is not part of the selected project.
        _createDraftCommand: null,

        // _projectService: [private] Array
        //      Local cache for notifications
        _notifications: null,

        constructor: function (params, projectService) {
            // summary:
            //      Constructs the ProjectNotification
            //
            // params: Object
            //      The keyword arguments
            //
            // projectService: ProjectService?
            //      ProjectService dependency, it is isn't provided it will load it from the epi/dependency

            this._projectService = projectService || dependency.resolve("epi.cms.ProjectService");

            this._createDraftCommand = new CreateDraft({ ignoreContentStatus: true });

            this.own(
                this._projectService.on("project-item-updated", lang.hitch(this, this._onItemUpdated)),
                this._projectService.on("project-item-removed", lang.hitch(this, this._onItemRemoved)),
                this._projectService.on("project-updated", function () {
                    this._onModelChanged(this.value);
                }.bind(this))
            );
        },

        _valueSetter: function (value) {
            // summary:
            //      Updates the notification when the property changes.
            // tags:
            //      private

            var contentModel = value.contentViewModel;
            this.value = contentModel;

            if (!contentModel || !contentModel.contentLink) {
                this.set("model", null);
                this._currentProjectItemId = 0;
                return;
            }

            this._createDraftCommand.set("model", contentModel);

            // Watch the model for changes to the content link since will occur when a new draft is created.
            var ownKey = "contentLinkWatch";
            this.destroyByKey(ownKey);
            this.ownByKey(ownKey, contentModel.watch("contentLink", lang.hitch(this, "_onModelChanged", contentModel)));

            // Invoke the model change callback since we have just set the model.
            this._notifications = {};
            this._onModelChanged(contentModel);
        },

        _onItemUpdated: function (item) {
            this._onItemsAddedOrRemoved(item.id);
        },

        _onItemRemoved: function (id) {
            this._onItemsAddedOrRemoved(id);
        },

        _onItemsAddedOrRemoved: function (projectItemId) {
            if (!this.model) {
                return;
            }

            if (this._currentProjectItemId === projectItemId) {
                this._projectService.getProjectItemsForContent(this.model.contentLink).then(function (projectItems) {
                    this._updateNotification(this.model.contentLink, projectItems);
                }.bind(this));
            }
        },

        _onModelChanged: function (contentModel) {
            // summary:
            //      Updates the notification when the model changes.
            // tags:
            //      private

            // If the model is suspended do not load or update the notification
            if (contentModel.get("isSuspended")) {
                return;
            }

            if (this._notifications && Object.prototype.hasOwnProperty.call(this._notifications, contentModel.contentLink)) {
                return;
            }

            this._projectService.getProjectItemsForContent(contentModel.contentLink)
                .then(function (projectItems) {
                    var languageContext = contentModel.languageContext,
                        isTranslationNeeded =
                                languageContext && languageContext.isTranslationNeeded &&
                                ApplicationSettings.currentContentLanguage === languageContext.preferredLanguage;

                    this.set("model", contentModel);
                    var projectItem = this._projectService.getCurrentProjectItemFromProjectItems(projectItems, contentModel.contentLink);
                    this._currentProjectItemId =  projectItem && projectItem.id || 0;

                    this._updateNotification(contentModel.contentLink, projectItems, contentModel.contentData.isPartOfAnotherProject, isTranslationNeeded);
                    this._notifications[contentModel.contentLink] = this._currentProjectItemId;
                }.bind(this))
                .otherwise(function () {
                    this._currentProjectItemId = 0;
                }.bind(this));
        },

        _updateNotification: function (contentLink, projectItems, isPartOfAnotherProject, isTranslationNeeded) {
            var self = this;

            this.destroyByKey(projectLinksKey);

            all([this._projectService.getProjects(projectItems), this._projectService.getCurrentProject()]).then(function (result) {
                var projects = result[0],
                    currentProject = result[1];

                if (projects.length === 0) {
                    if (isPartOfAnotherProject) {
                        this.set("notification", null);
                        return;
                    }

                    self._checkAnotherVersionInProject(contentLink);
                    return;
                }

                var contentEl = document.createElement("div");
                contentEl.appendChild(createProjectIconEl());

                //Create the part of project text
                var projectContainerEl = createMessageContainerEl();
                contentEl.appendChild(projectContainerEl);

                if (isPartOfAnotherProject) {
                    var partOfAnotherProjectEl = document.createTextNode(resources.partofanotherproject + ": ");
                    projectContainerEl.appendChild(partOfAnotherProjectEl);
                    self._createClickableProjectsLinks(projectContainerEl, projects);
                } else if (currentProject) {
                    var currentProjectLinkHtml = "<a class=\"dijitReset dijitButtonText\" tabIndex=\"0\">" + resources.current + "</a>";
                    var currentProjectHtml = lang.replace(resources.partofcurrentproject, {
                        currentPlaceholder: currentProjectLinkHtml
                    });

                    projectContainerEl.innerHTML = currentProjectHtml;
                    self._attachProjectNameClickEvent(projectContainerEl.querySelector("a"), currentProject);
                } else {
                    var partOfProjectEl = document.createTextNode(resources.partofproject + ": ");
                    projectContainerEl.appendChild(partOfProjectEl);
                    self._createClickableProjectsLinks(projectContainerEl, projects);
                }

                var commands =
                        isPartOfAnotherProject && self._projectService.isProjectModeEnabled && !isTranslationNeeded ?
                            [self._createDraftCommand] : [];

                self.set("notification", {
                    content: contentEl,
                    commands: commands
                });
            });
        },

        _checkAnotherVersionInProject: function (contentLink) {
            // summary:
            //      Check if another content version is a part of the project
            // tags:
            //      private

            contentLink = (new ContentReference(contentLink)).createVersionUnspecificReference().toString();

            when(this._projectService.getProjectsForContent(contentLink)).then(function (projects) {
                if (projects.length === 0) {
                    this.set("notification", null);
                    return;
                }

                // remove duplicated projects
                var projectIds = projects.map(function (project) {
                    return project.id;
                });
                projects = projects.filter(function (project, index) {
                    return projectIds.indexOf(project.id) === index;
                });

                var contentEl = document.createElement("div");
                contentEl.appendChild(createProjectIconEl());

                var projectContainerEl = createMessageContainerEl();
                contentEl.appendChild(projectContainerEl);

                var contentText = document.createTextNode(resources.anotherversioninproject + " ");
                projectContainerEl.appendChild(contentText);

                this._createClickableProjectsLinks(projectContainerEl, projects);

                this.set("notification", {
                    content: contentEl,
                    commands: []
                });
            }.bind(this));
        },

        _createClickableProjectsLinks: function (contentEl, projects) {
            projects.forEach(function (project, index) {
                var projectEl = document.createElement("a");
                projectEl.setAttribute("tabIndex", 0);
                projectEl.classList.add("dijitReset", "dijitButtonText");
                var projectName = document.createTextNode(project.name);
                projectEl.appendChild(projectName);
                contentEl.appendChild(projectEl);

                if (index < projects.length - 1) {
                    var separatorText = document.createTextNode(", ");
                    contentEl.appendChild(separatorText);
                }
                this._attachProjectNameClickEvent(projectEl, project);
            }, this);
        },

        _attachProjectNameClickEvent: function (projectEl, project) {
            this.ownByKey(projectLinksKey, on(projectEl, "click", this._onProjectNameClick.bind(this, project.id)));
            this.ownByKey(projectLinksKey, on(projectEl, "keypress", function (event) {
                if (event.keyCode === keys.ENTER) {
                    this._onProjectNameClick(project.id);
                }
            }.bind(this)));
        },

        _onProjectNameClick: function (projectId) {
            //Publishes request change for project overview
            var newProjectUri = "epi.cms.project:///" + projectId;
            topic.publish("/epi/shell/context/request", { uri: newProjectUri }, { sender: null });
        }
    });
});
